package com.persagy.proxy.adm.utils;

import cn.hutool.core.collection.CollUtil;
import com.fasterxml.jackson.databind.node.ObjectNode;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.locationtech.jts.geom.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @ClassName GeoToolsUtil
 * @Description: 点面包含关系分析
 * @Author linhuili
 * @Date 2021/9/3 20:31
 * @Version V1.0
 **/
public class GeoToolsUtil {
    /**
     * 创建点
     * @param pointLocal
     */
    private static Point createPoint(ObjectNode pointLocal){
        Long x =0L;
        Long y =0L;
        Long z =0L;
        if(pointLocal.get("x") != null){
            x = pointLocal.get("x").longValue();
        }
        if(pointLocal.get("y") != null){
            y = pointLocal.get("y").longValue();
        }
        if(pointLocal.get("z") !=null){
            z = pointLocal.get("z").longValue();
        }
        GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory();
        Coordinate coord = new Coordinate(x, y,z);
        Point point = geometryFactory.createPoint(coord);
        return point;
    }

    /**
     * 创建面
     * @param outLines
     */
    private static Polygon createPolygon(List<ObjectNode> outLines){
        GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory();
        List<Coordinate> objects = new ArrayList<>();
        for (ObjectNode outLine : outLines) {
            Long x = 0L;
            Long y = 0L;
            Long z = 0L;
            if(outLine.get("x") != null){
                x =outLine.get("x").longValue();
            }
            if(outLine.get("y") != null){
                y = outLine.get("y").longValue();
            }
            if(outLine.get("z") !=null){
                z = outLine.get("z").longValue();
            }
            Coordinate coordinate = new Coordinate(x, y, z);
            objects.add(coordinate);
        }
        //注意数据的闭合
        Coordinate[] coordinates = objects.toArray(new Coordinate[0]);
        LinearRing ring = geometryFactory.createLinearRing(coordinates);
        LinearRing holes[] = null;
        Polygon polygon = geometryFactory.createPolygon(ring, holes);

        return polygon;
   }

    /**
     * 点面包含关系判断
     * 用途：判断一个面是否包含一个点，即一个点是否在一个面内
     * @param pointLocal 坐标点
     * @param outLines 轮廓线
     * @return
     */
   public static Boolean isPointInPoly(ObjectNode pointLocal, List<ObjectNode> outLines) {
       //创建点
       Point point = createPoint(pointLocal);
       //创建面
       Polygon polygon = createPolygon(outLines);
       //判断点面包含关系
       boolean flag = polygon.contains(point);
       return flag;
   }


    /**
     * 点面包含关系判断
     * 用途：判断一个面是否包含一个点，即一个点是否在一个面内
     * @param fromOutLines: 轮廓坐标
     * @param toOutLines: 轮廓坐标
     * @return
     */
    public static Boolean isPolyInPoly(List<ObjectNode> fromOutLines,
                                       List<ObjectNode> toOutLines) {
        if (CollUtil.isEmpty(fromOutLines) || CollUtil.isEmpty(toOutLines)){
            return false;
        }
        //创建点
        Polygon fromPolygon = createPolygon(fromOutLines);
        //创建面
        Polygon toPolygon = createPolygon(toOutLines);
        //判断面-面包含关系
        return fromPolygon.contains(toPolygon)
                || fromPolygon.equalsTopo(toPolygon)
                || fromPolygon.overlaps(toPolygon)
                || toPolygon.contains(fromPolygon);
    }

}
