package com.persagy.proxy.object.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.proxy.adm.constant.GraphCodeEnum;
import com.persagy.proxy.adm.constant.RelCodeEnum;
import com.persagy.proxy.adm.service.impl.AbstractAdmBaseServiceImpl;
import com.persagy.proxy.adm.utils.AdmContextUtil;
import com.persagy.proxy.object.model.AdmFloor;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.object.model.AdmSpace;
import com.persagy.proxy.object.service.IAdmFloorService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ADM 楼层 实现类
 * @author lvxy
 * @date 2021/8/16
 */
@Service
public class AdmFloorServiceImpl extends AbstractAdmBaseServiceImpl<AdmFloor> implements IAdmFloorService {
    @Autowired
    private ObjectMapper objectMapper;

    public static final Boolean QUERY = true;
    public static final Boolean CREAT_UPDATE = false;
    /***
     * Description: 根据条件查询
     * @param criteria : 请求参数
     * @return : com.persagy.proxy.adm.request.AdmResponse
     * @author : lijie
     * @date :2021/10/28 20:13
     * Update By lijie 2021/10/28 20:13
     */
    @Override
    public AdmResponse query(AdmQueryCriteria criteria) {
        // 1.查询楼层
        InstanceUrlParam context = AdmContextUtil.toDmpContext();
        criteria.setName(AdmFloor.OBJ_TYPE);
        // 设置级联查询参数
        AdmQueryCriteria buildCriteria = new AdmQueryCriteria();
        buildCriteria.setName("building");
        criteria.setCascade(CollUtil.newArrayList(buildCriteria));
        AdmResponse response = this.doQuery(context, criteria, AdmFloor.class);
        List<AdmFloor> admFloors = (List<AdmFloor>) response.getContent();
        if (CollUtil.isNotEmpty(admFloors)){
            admFloors.forEach(admFloor -> {
                if (null==admFloor.getBuilding()){
                    return;
                }
                admFloor.setBuildingId(admFloor.getBuilding().getId());
            });
        }
        // 处理properteies 特殊属性
        ensureSpecialValues(context, admFloors, QUERY);
        return response;
    }
    /***
     * Description: 子类重写,以便处理带buildingId
     * @param dmpRequest : 中台查询参数
     * @return : void
     * @author : lijie
     * @date :2021/10/28 20:25
     * Update By lijie 2021/10/28 20:25
     */
    @Override
    protected void processDmpCriteria(QueryCriteria dmpRequest) {
        ObjectNode criteria = dmpRequest.getCriteria();
        if (null==criteria){
            return;
        }
        if (criteria.has("buildingId")){
            // 有建筑id过滤
            ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
            ObjectNode relationTo = objectMapper.createObjectNode();
            relationTo.put("graphCode", GraphCodeEnum.ArchSubset.name());
            relationTo.put("relCode", RelCodeEnum.Bd2Fl.name());
            relationTo.set("objFrom",criteria.get("buildingId"));
            criteria.set("relationTo",relationTo);
            criteria.remove("buildingId");
        }
    }

    @Override
    public AdmResponse doQueryOutLine(InstanceUrlParam context, AdmQueryCriteria request, Class<AdmFloor> clazz) {
        request.setProjection(null);
        List<String> list = new ArrayList<>();
        list.add("id");
        list.add("name");
        list.add("localId");
        list.add("localName");
        list.add("classCode");
        list.add("outline");
        list.add("floorSequenceId");
        list.add("modelId");
        list.add("floorElevation");
        list.add("floorMap");
        list.add("floorSequenceID");
        list.add("height");
        list.add("infos");
        list.add("properties");
        request.setProjection(list);
        AdmResponse rs = query(request);
        List<AdmFloor> vos = (List<AdmFloor>) rs.getContent();
        ensureSpecialValues(context, vos, QUERY);
        vos.forEach(vo->{
            Map<String, Object> objectMap = MapUtil.defaultIfEmpty(vo.getInfos(), new HashMap<>());
            objectMap.put("floorSequenceID",vo.getFloorSequenceID());
            vo.setFloorSequenceID(null);
            vo.setInfos(objectMap);
        });
        rs.setContent(vos);
        return rs;
    }

    @Override
    public List<AdmFloor> queryFloorInBuilding(InstanceUrlParam context, String buildingId) {
        AdmQueryCriteria request = new AdmQueryCriteria();
        request.setName(AdmFloor.OBJ_TYPE);
        request.resetRelationCond(false, "ArchSubset", "Bd2Fl", buildingId);
        request.setOrders("floorSequenceId desc");
        AdmResponse rs = query(request);
        List<AdmFloor> vos = (List<AdmFloor>) rs.getContent();
        ensureSpecialValues(context, vos, QUERY);
        return vos;
    }

    /**
     * 楼层自定义信息点处理 新增，properties 转化为String类型，放到propertiesInfo信息点存储，查询相关，propertiesInfo转为为properties 页面展示
     * @param context
     * @param queryFlag true 查询，false 更新或者新增
     * @return
     */
    @Override
    public void ensureSpecialValues(InstanceUrlParam context, List<AdmFloor> vos, boolean queryFlag) {
        if(CollUtil.isNotEmpty(vos)){
            if(queryFlag){//查询
                vos.stream().forEach(admFloor -> {
                    if(StrUtil.isNotEmpty(admFloor.getPropertiesInfo())){
                        try {
                            ObjectNode properties = (ObjectNode) objectMapper.readTree(admFloor.getPropertiesInfo());
                            admFloor.setProperties(properties);
                            admFloor.setPropertiesInfo(null);
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                });
            }else{//新增或者更新
                vos.stream().forEach(admFloor -> {
                    if(!JSONUtil.isNull(admFloor.getProperties())){
                        admFloor.setPropertiesInfo(admFloor.getProperties().toString());
                        admFloor.setProperties(null);
                    }
                });
            }
        }
    }
}