package com.persagy.proxy.report.controller;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ResourceUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.context.AppContext;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.define.client.DigitalDefineFacade;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import com.persagy.dmp.define.entity.ObjectTypeDefine;
import com.persagy.dmp.digital.client.DigitalObjectFacade;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.constant.AdmObjectType;
import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.adm.handler.RelationReportHandler;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.strategy.RelationObjectContext;
import com.persagy.proxy.adm.utils.AdmExcelUtil;
import com.persagy.proxy.adm.utils.AdmQueryCriteriaHelper;
import com.persagy.proxy.report.model.AdmRelationObject;
import com.persagy.proxy.report.model.EquipmentBindPointExcel;
import com.persagy.proxy.report.model.EquipmentCountExcel;
import com.persagy.proxy.report.model.EquipmentExcel;
import com.persagy.proxy.report.service.IRelationReportService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * 报表/模板下载
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021年8月30日 下午7:41:56
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@SuppressWarnings("deprecation")
public class RelationReportController {

	public static final String EXPORT_SHEET_NAME = "关系维护";
	
	@Value("${middleware.group.code}")
	private String defaultCode;
	
	private final RelationReportHandler relationReportHandler;
	
	private final RelationObjectContext relationObjectContext;
	
	private final IRelationReportService relationReportService;
	
	/**
	 * 查询总数量
	 * 
	 * @param queryCriteria
	 * @param request
	 * @return
	 */
	@RequestMapping("/graphic/query-count")
	public AdmResponse queryCount(HttpServletRequest request) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		
		List<JSONObject> relationProjectCal = this.relationReportService.getAllRelationProjectCalTree(groupCode, projectId);
		return AdmResponse.success(relationProjectCal);
	}
	
	/**
	 * 查询总览数量-根据类型查询关系数量
	 * 
	 * @param type	relationType
	 */
	@RequestMapping("/graphic/type/query-count")
	public AdmResponse countRelationObjects(@RequestParam String type, HttpServletRequest request) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		JSONObject objectNode = this.relationReportService.getRelationProjectCal(groupCode, projectId, type);
 		return AdmResponse.success(Lists.newArrayList(objectNode));
	}
	
	/**
	 * 查询关系总览（空表，先插入）
	 * 
	 * @param content:模糊查询卡片名称关键字
	 * @return
	 */
	@RequestMapping("/graphic/overview")
	public JSONObject overview(@RequestParam(required = false) String content) throws JsonProcessingException {
		JSONObject overview = this.relationReportService.overview(content);
		return overview;
	}
	
	/**
	 * 项目关系类型查询
	 * {"filters":"relationType in [\"bd2sp\",\"fl2sp\"];computationalState>1"}
	 * 
	 * @param queryCriteria
	 * @param request
	 * @return
	 */
	@RequestMapping("/graphic/relation_type_project")
	public AdmResponse queryRelationObjects(@RequestBody AdmQueryCriteria requestCriteria, HttpServletRequest request) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		String filters = requestCriteria.getFilters();
		
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		ObjectNode criteria = null;
		Set<String> graphs = new HashSet<String>();
		Set<String> relCodes = new HashSet<String>();
		if (StrUtil.isNotBlank(filters)) {
			criteria = AdmQueryCriteriaHelper.toCriteria(null, requestCriteria.getFilters());
			
			if (filters.contains("relationType")) {
				String condition = filters.substring(filters.indexOf("in [") + 4, filters.lastIndexOf("]"));
				String[] split = condition.split(",");
				for (String relType : split) {
					relType = relType.replaceAll("\"", "");
					AdmRelationTypeEnum typeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
					if (typeEnum != null) {
						graphs.add(typeEnum.getGraphCode());
						relCodes.add(typeEnum.getRelCode());
					}
				}
				criteria.remove("relationType");
			}
			if (criteria.has("automatic")) {
				criteria.put("automaticFlag", criteria.get("automatic"));
				criteria.remove("automatic");
			}
			if (criteria.has("source")) {
				criteria.put("sourceFlag", criteria.get("source"));
				criteria.remove("source");
			}
		}
		if (criteria == null) {
			criteria = objectMapper.createObjectNode();
		}
		if (CollectionUtil.isNotEmpty(graphs)) {
			JSONObject graphObject = new JSONObject();
			graphObject.put("$in", graphs);
			criteria.putPOJO("graphCode", graphObject);
		}
		if (CollectionUtil.isNotEmpty(relCodes)) {
			JSONObject relObject = new JSONObject();
			relObject.put("$in", relCodes);
			criteria.putPOJO("relCode", relObject);
		}
		criteria.put("projectId", projectId);
		queryCriteria.setCriteria(criteria);
		List<JSONObject> relationProjectCal = this.relationReportService.getAllRelationProjectCal(queryCriteria, groupCode, projectId);
		return AdmResponse.success(relationProjectCal);
	}
	
	/**
	 * 下载报告-查询key值
	 * 
	 * @param queryCriteria
	 * @param request
	 * @return
	 */
	@RequestMapping("/graphic/downloads/point/key")
	public AdmResponse downloadsPointKey(@RequestBody AdmQueryCriteria queryCriteria, HttpServletRequest request) {
		//String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		//String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		//List<ObjectNode> relationProjectCal = this.relationReportService.getAllRelationProjectCal(groupCode, projectId);
		return AdmResponse.success(new ArrayList<>());
	}
	
	/**
	 * 下载报告-设备-未关联空间的设备
	 * 设备.xlsx 是个空模板，所以这里未使用 
	 * 
	 * @param projectId
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/graphic/downloads/eq/not/sp")
	public void downloadsEqNotSp(@RequestParam(required = false) String projectId, HttpServletRequest request, HttpServletResponse response) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		if (StrUtil.isBlank(projectId)) {
			projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		}
		try {
            response.setContentType("application/vnd.ms-excel");
            String encode = StandardCharsets.UTF_8.name();
            response.setCharacterEncoding(encode);
            // 防止中文乱码
            String fileName = URLEncoder.encode("equipment", encode);
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            
            List<EquipmentExcel> equipByNotSpace = this.relationReportHandler.findEquipByNotSpace(groupCode, projectId);
            EasyExcel.write(response.getOutputStream(), EquipmentExcel.class).sheet("设备").doWrite(equipByNotSpace);
		} catch(Exception e) {
			log.error("设备数据导出异常", e);
		}
	}
	
	/**
	 * 下载报告-统计项目下已有的设备，信息点，每个设备
	 * 设备.xlsx 是个空模板，所以这里未使用 
	 * 
	 * @param projectId
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/graphic/downloads/project/binding/point")
	public void downloadsProjectBindingPoint(@RequestParam(required = false) String projectId, HttpServletRequest request, HttpServletResponse response) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		if (StrUtil.isBlank(projectId)) {
			projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		}
		try {
            response.setContentType("application/vnd.ms-excel");
            String encode = StandardCharsets.UTF_8.name();
            response.setCharacterEncoding(encode);
            // 防止中文乱码
            String fileName = URLEncoder.encode("设备实例交付", encode);
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            
            List<EquipmentBindPointExcel> equipmentBindPointExcel = this.relationReportHandler.queryProjectBindPoint(groupCode, projectId);
            EasyExcel.write(response.getOutputStream(), EquipmentBindPointExcel.class).sheet(0).doWrite(equipmentBindPointExcel);
		} catch(Exception e) {
			log.error("设备数据导出异常", e);
		}
	}
	
	/**
	 * 下载报告-统计项目下已有的设备，以及静态和iot 信息点使用情况
	 * 设备.xlsx 是个空模板，所以这里未使用 
	 * 
	 * @param projectId
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/graphic/downloads/project/equip/count")
	public void downloadsProjectEquipPoint(@RequestParam(required = false) String projectId, HttpServletRequest request, HttpServletResponse response) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		if (StrUtil.isBlank(projectId)) {
			projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		}
		try {
            response.setContentType("application/vnd.ms-excel");
            String encode = StandardCharsets.UTF_8.name();
            response.setCharacterEncoding(encode);
            // 防止中文乱码
            String fileName = URLEncoder.encode("equipment", encode);
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            
            List<EquipmentCountExcel> equipmentCountExcel = this.relationReportHandler.countClassCodeEquip(groupCode, projectId);
            EasyExcel.write(response.getOutputStream(), EquipmentCountExcel.class).sheet(0).doWrite(equipmentCountExcel);
		} catch(Exception e) {
			log.error("设备数据导出异常", e);
		}
	}
	
	/**
	 * 下载报告-设备静态信息维护模板
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/downloads/equip/template")
	public void downloadEquipTemplate(HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			// 防止中文乱码
			String fileName = URLEncoder.encode("台账信息导入模板", encode);
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
			
			File templateFile = AdmExcelUtil.getTemplateFile("equip_template.xlsx");
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(templateFile).build();
		} catch (Exception e) {
			log.error("设备静态信息维护模板下载失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}

	/**
	 * 下载报告-点表
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/downloads/point")
	public void downloadPoint(@RequestParam(required = false) String key, HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			response.setHeader("Content-disposition", "attachment;filename=" + key);
			
			File file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + AdmExcelUtil.TEMP_UPLOAD_PAHT + key);
			if (!file.exists()) {
				file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + AdmExcelUtil.TEMP_PAHT + "relation-template.xlsx");
			}
			
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(file).build();
		} catch (Exception e) {
			log.error("下载报告-点表失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 下载模板
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/template-downloads")
	public void templateDownloads(HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			response.setHeader("Content-disposition", "attachment;filename=relation-template.xlsx");
			
			File templateFile = AdmExcelUtil.getTemplateFile("relation-template.xlsx");
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(templateFile).build();
		} catch (Exception e) {
			log.error("模板下载失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 下载报告
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/report-downloads")
	public void reportDownloads(@RequestParam String projectId, @RequestParam String relType, @RequestParam(required = false) String zoneType, 
			HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			
			String templateFile = projectId + AdmCommonConstant.LINE_THROUGH + relType + AdmCommonConstant.EXCEL_SUFFIX_XLSX;
			response.setHeader("Content-disposition", "attachment;filename=" + templateFile);
			
			File file = AdmExcelUtil.getTemplateOrDefaultFile(templateFile);
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(file).build();
		} catch (Exception e) {
			log.error("下载报告失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 导出-点位模板
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/export/point")
	public void exportPoint(@RequestParam String buildingId, @RequestParam String floorId, @RequestParam String category, 
			HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			
			//excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(file).build();
		} catch (Exception e) {
			log.error("导出-点位模板失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 导出关系
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/export")
	public void export(@RequestParam String projectId, @RequestParam String relType, @RequestParam(required = false) String code, 
			@RequestParam(required = false) String zoneType, HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
			groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
			code = StrUtil.isBlank(code) ? "对象ID" : code;
			
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			response.setHeader("Content-disposition", "attachment;filename=relation-template.xlsx");
			
			log.info("下载报表： groupCode[{}], projectId[{}], relType[{}], code[{}]", groupCode, projectId, relType, code);
			AdmRelationTypeEnum typeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
			
			List<AdmRelationObject> relationObjects = null;
			if (typeEnum != null) {
				String graphAndRelKey = typeEnum.getGraphCode() + AdmCommonConstant.UNDERLINE + typeEnum.getRelCode();
				relationObjects = this.relationObjectContext.exportRelationObject(groupCode, projectId, graphAndRelKey);
			}
			
			File templateFile = AdmExcelUtil.getTemplateFile("relation-template.xlsx");
			EasyExcel.write(response.getOutputStream()).withTemplate(templateFile).sheet(EXPORT_SHEET_NAME).needHead(false).doWrite(relationObjects);
		} catch (Exception e) {
			log.error("深化点导入失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	
	/************************************************** 文件上传 ***********************************************/
	
	/**
	 * 文件导入，这里由于需要回写文件，且存储到本地，采用原始POI文件解析、上传、回写、存储
	 * 
	 * @param relType
	 * @param code
	 * @param zoneType
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/import")
	public AdmResponse importFile(@RequestParam("file") MultipartFile file, @RequestParam String relType, 
			@RequestParam(required = false) String zoneType, HttpServletRequest request, HttpServletResponse response) {
		String failure = null;
		XSSFWorkbook workbook = null;
		FileOutputStream fileOutputStream = null;
		try {
			if (file == null) {
				return AdmResponse.failure("未选择文件");
			}
			AdmRelationTypeEnum typeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
			if (typeEnum == null) {
				return AdmResponse.failure("不支持的关系类型");
			}
			String graphAndRelKey = typeEnum.getGraphCode() + AdmCommonConstant.UNDERLINE + typeEnum.getRelCode();
			
			int failurNum = 0;
			int successNum = 0;
			String graphId = null;
			// 临时存储行号
			Set<Integer> rowNums = new HashSet<Integer>();
			// 批量更新数据
			List<ObjectNode> relationObjects = new ArrayList<ObjectNode>();
			
			String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
			groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
			String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
			
			workbook = new XSSFWorkbook(file.getInputStream());
		    XSSFSheet sheet = workbook.getSheet("关系维护");
		    // 获取识别码的属性
		    String code = sheet.getRow(2).getCell(1).toString();
		    int lastRowNum = sheet.getLastRowNum();
		    for (int i = 5; i <= lastRowNum; i++) {
		    	XSSFRow xssfRow = sheet.getRow(i);
		    	if (xssfRow == null) {
					continue;
				}
		    	// 封装cell数据
		    	XSSFCell masterCodeCell = xssfRow.getCell(0);
		    	XSSFCell slaveCodeCell = xssfRow.getCell(8);
		    	XSSFCell masterIdCell = xssfRow.getCell(3);
		    	XSSFCell slaveIdCell = xssfRow.getCell(11);
		    	
		    	AdmRelationObject relationObject = new AdmRelationObject();
		    	relationObject.setMasterCode(masterCodeCell == null ? null : masterCodeCell.getStringCellValue());
		    	relationObject.setSlaveCode(slaveCodeCell == null ? null : slaveCodeCell.getStringCellValue());
		    	relationObject.setMasterId(masterIdCell == null ? null : masterIdCell.getStringCellValue());
		    	relationObject.setSlaveId(slaveIdCell == null ? null : slaveIdCell.getStringCellValue());
		    	
		    	// 校验数据
				Object result = this.relationObjectContext.checkRelationObject(relationObject, groupCode, projectId, graphAndRelKey, code, typeEnum);
				if (result instanceof String) {
					failurNum++;
					xssfRow.createCell(16).setCellValue((String)result);
					continue;
				}
				ObjectNode relation = (ObjectNode) result;
				graphId = relation.get("graphId") == null ? graphId : relation.get("graphId").asText();
				
				// 每一千条数据，批量新增一次
				rowNums.add(i);
				relationObjects.add(relation);
				if (relationObjects.size() >= 1000) {
					boolean objects = this.relationObjectContext.saveRelationObjects(relationObjects, groupCode, projectId, graphAndRelKey);
					if (objects) {
						successNum += relationObjects.size();
						for (int rowNum : rowNums) {
							sheet.getRow(rowNum).createCell(16).setCellValue("关联成功");
						}
					} else {
						failurNum += relationObjects.size();
						for (int rowNum : rowNums) {
							sheet.getRow(rowNum).createCell(16).setCellValue("关联失败");
						}
					}
					rowNums.clear();
					relationObjects.clear();
				}
			}
		    // 最后一次遗留数据录入
		    if (CollectionUtil.isNotEmpty(relationObjects)) {
		    	boolean objects = this.relationObjectContext.saveRelationObjects(relationObjects, groupCode, projectId, graphAndRelKey);
				if (objects) {
					successNum += relationObjects.size();
					for (int rowNum : rowNums) {
						sheet.getRow(rowNum).createCell(16).setCellValue("关联成功");
					}
				} else {
					failurNum += relationObjects.size();
					for (int rowNum : rowNums) {
						sheet.getRow(rowNum).createCell(16).setCellValue("关联失败");
					}
				}
			}
		    
        	try {
        		// 调用中台存储完成之后，此文件存储至本地
        		String fileName = projectId + AdmCommonConstant.LINE_THROUGH + relType + AdmCommonConstant.EXCEL_SUFFIX_XLSX;
        		File templateFile = AdmExcelUtil.createTemplateFile(AdmCommonConstant.SERVER_ROOT_PATH + AdmExcelUtil.TEMP_PAHT, fileName);
        		fileOutputStream = new FileOutputStream(templateFile);
        		workbook.write(fileOutputStream);
			} catch (Exception e) {
				log.error("文件存储失败", e);
			}
        	
        	return AdmResponse.success(Lists.newArrayList(AdmExcelUtil.getImportResult(graphId, successNum, failurNum)));
		} catch (Exception e) {
			log.error("文件导入失败", e);
			failure = e.getMessage();
		} finally {
			if (fileOutputStream != null) {
				try {
					fileOutputStream.flush();
					fileOutputStream.close();
				} catch (IOException e) {}
			}
			if (workbook != null) {
				try {
					workbook.close();
				} catch (IOException e) {}
			}
		}
		return AdmResponse.failure(failure);
	}
	
	/**
	 * 深化点信息导入
	 * 
	 * @param file
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/graphic/import/point")
	public AdmResponse importFile(@RequestParam("file") MultipartFile file, HttpServletRequest request, HttpServletResponse response) {
		int failCount = 0;
		int successCount = 0;
		String failure = null;
		XSSFWorkbook workbook = null;
		String groupCode = AppContext.getContext().getGroupCode();
		String projectId = AppContext.getContext().getProjectId();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		try {
			if (file == null) {
				return AdmResponse.failure("未选择文件");
			}
			
			workbook = new XSSFWorkbook(file.getInputStream());
		    XSSFSheet sheet = workbook.getSheetAt(0);
		    XSSFRow row = sheet.getRow(0);
		    if (row == null) {
		    	return AdmResponse.failure("文件标题行不存在");
			}
		    
		    int identificationCode = -1, infosCode = -1, tableCode = -1, functionCode = -1, infosFeedback = -1, 
		    		classCodeIndexes = -1, bimIdIndexes = -1;
		    
		    short lastCellNum = row.getLastCellNum();
		    for (int i = 0; i <= lastCellNum; i++) {
				XSSFCell cell = row.getCell(i);
				if (cell != null && cell.getStringCellValue() != null && "本地编码".equals(cell.getStringCellValue().trim())) {
                    identificationCode = i;
                } else if (cell != null && cell.getStringCellValue() != null && "信息点编码".equals(cell.getStringCellValue().trim())) {
                    infosCode = i;
                } else if (cell != null && cell.getStringCellValue() != null && "表号".equals(cell.getStringCellValue().trim())) {
                    tableCode = i;
                } else if (cell != null && cell.getStringCellValue() != null && "功能号".equals(cell.getStringCellValue().trim())) {
                    functionCode = i;
                } else if (cell != null && cell.getStringCellValue() != null && "反馈信息".equals(cell.getStringCellValue().trim())) {
                    infosFeedback = i;
                } else if (cell != null && cell.getStringCellValue() != null && "设备类型".equals(cell.getStringCellValue().trim())) {
                    classCodeIndexes = i;
                } else if (cell != null && cell.getStringCellValue() != null && "BIMID".equals(cell.getStringCellValue().trim())) {
                    bimIdIndexes = i;
                }
			}
		    
			if (identificationCode == -1) {
				failure = "缺少识别码：本地编码";
			} else if (infosCode == -1) {
				failure = "缺少信息点编码";
			} else if (tableCode == -1) {
				failure = "缺少表号";
			} else if (functionCode == -1) {
				failure = "缺少功能号";
			} else if (infosFeedback == -1) {
				failure = "缺少反馈信息";
			} else if (classCodeIndexes == -1) {
				failure = "缺少设备类型";
			} else if (bimIdIndexes == -1) {
				failure = "缺少BIMID";
			}
			if (StrUtil.isNotBlank(failure)) {
				return AdmExcelUtil.getImportFailResult(failure, "3", 0, 0);
			}
			
			// 业务数据获取
		    int lastRowNum = sheet.getLastRowNum();
		    for (int i = 1; i <= lastRowNum; i++) {
		    	XSSFRow xssfRow = sheet.getRow(i);
		    	if (xssfRow == null) {
		    		sheet.getRow(0).createCell(16).setCellValue("第" + i + " 行数据为空");
					continue;
				}
		    	XSSFCell identificationCodeCell = xssfRow.getCell(identificationCode);
		    	if (identificationCodeCell == null || StrUtil.isBlank(identificationCodeCell.getStringCellValue())) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("本地编码不可以为空");
                    continue;
				}
		    	identificationCodeCell.setCellType(CellType.STRING);
		    	XSSFCell infosCodeCell = xssfRow.getCell(infosCode);
		    	if (infosCodeCell == null || StrUtil.isBlank(infosCodeCell.getStringCellValue())) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("信息点编码不可以为空");
                    continue;
				}
		    	infosCodeCell.setCellType(CellType.STRING);
		    	XSSFCell tableCodeCell = xssfRow.getCell(tableCode);
		    	if (tableCodeCell == null || StrUtil.isBlank(tableCodeCell.getRawValue())) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("表号不可以为空");
                    continue;
				}
		    	tableCodeCell.setCellType(CellType.STRING);
		    	XSSFCell functionCodeCell = xssfRow.getCell(functionCode);
		    	if (functionCodeCell == null || StrUtil.isBlank(functionCodeCell.getRawValue())) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("功能号不可以为空");
                    continue;
				}
		    	functionCodeCell.setCellType(CellType.STRING);
		    	XSSFCell classCodeIndexesCell = xssfRow.getCell(classCodeIndexes);
		    	if (classCodeIndexesCell == null || StrUtil.isBlank(classCodeIndexesCell.getStringCellValue())) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("设备类型不可以为空");
                    continue;
				}
		    	classCodeIndexesCell.setCellType(CellType.STRING);
		    	String classCode = classCodeIndexesCell.getStringCellValue().trim();
		    	// 验证此classCode是否存在
		    	ObjectTypeDefine typeDefine = this.queryClassList(groupCode, projectId, classCode, AdmObjectType.SYSTEM.getIndex());
		    	if (typeDefine == null) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("设备类型未在数据字典定义范围");
                    continue;
				}
		    	
		    	// 获取实际cell值
		    	String val0 = identificationCodeCell.getStringCellValue().trim();
		    	String val1 = infosCodeCell.getStringCellValue().trim();
		    	String val2 = tableCodeCell.getStringCellValue().trim();
		    	String val3 = functionCodeCell.getStringCellValue().trim();
		    	
		    	// 获取设备信息
		    	ObjectNode criteria = objectMapper.createObjectNode();
		    	criteria.put("localId", val0);
		    	criteria.put("classCode", classCode);
		    	List<ObjectNode> equipments = this.relationReportService.getObjectNode(criteria, groupCode, projectId, null);
		    	if (CollectionUtil.isEmpty(equipments)) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("设备类型下本地编码不存在");
                    continue;
				} else if (equipments.size() > 1) {
					String temp = null;
					for (ObjectNode objectNode : equipments) {
						String bimId = this.getObjectBimId(objectNode);
						if (StrUtil.isNotBlank(bimId)) {
							temp = (StrUtil.isBlank(temp) ? "" : (temp + ",")) + bimId;
						}
					}
					failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("本地编码重复");
		    		if (StrUtil.isNotBlank(temp)) {
		    			xssfRow.createCell(bimIdIndexes).setCellValue(temp);
					}
                    continue;
				}
		    	
		    	// 获取信息点定义
		    	List<ObjectInfoDefine> funidList = this.relationReportService.queryFunidList(groupCode, projectId, Sets.newHashSet(classCode), val1, null);
		    	if (CollectionUtil.isEmpty(funidList)) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("信息点不存在");
                    continue;
				}
		    	ObjectInfoDefine infoDefine = funidList.get(0);
		    	
		    	 // 数据更新，更新对应code值为，表号和功能号
		    	ObjectNode objectNode = objectMapper.createObjectNode();
		    	objectNode.put("id", equipments.get(0).get("id").asText());
		    	objectNode.put(infoDefine.getCode(), val2 + "-" +val3);
		    	
		    	ObjectDigital updateOne = DigitalObjectFacade.updateOne(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, objectNode);
		    	if (updateOne == null) {
		    		failCount++;
		    		xssfRow.createCell(infosFeedback).setCellValue("失败");
		    		String bimId = this.getObjectBimId(equipments.get(0));
		    		if (StrUtil.isNotBlank(bimId)) {
		    			xssfRow.createCell(bimIdIndexes).setCellValue(bimId);
					}
				} else {
					xssfRow.createCell(infosFeedback).setCellValue("成功");
					successCount++;
				}
			}

		    // 文件存储及返回
		    String fileName = AdmExcelUtil.writeFile2Local(workbook, projectId, AdmCommonConstant.SERVER_ROOT_PATH + AdmExcelUtil.TEMP_UPLOAD_PAHT, file.getOriginalFilename());
		    return AdmExcelUtil.getImportSuccessResult(fileName, successCount, failCount);
		} catch (Exception e) {
			log.error("文件导入失败", e);
			failure = e.getMessage();
		} finally {
			if (workbook != null) {
				try {
					workbook.close();
				} catch (IOException e) {}
			}
		}
		return AdmResponse.failure(failure);
	}
	
	
	/***************************************************************************************************************/
	
	/**
	 * 获取类定义信息
	 * 
	 * @param groupCode
	 * @param projectId
	 * @param classCode
	 * @param objType 不等于条件
	 * @return
	 */
	private ObjectTypeDefine queryClassList(String groupCode, String projectId, String classCode, String objType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria classCriteria = new QueryCriteria();
		ObjectNode condition = objectMapper.createObjectNode();
		if (StrUtil.isNotBlank(classCode)) {
			condition.put("code", classCode);
		}
		if (StrUtil.isNotBlank(objType)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$ne", objType);
			condition.putPOJO("objType", arrays);
		}
		classCriteria.setCriteria(condition);
		List<ObjectTypeDefine> objectNodes = DigitalDefineFacade.queryObjectType(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, classCriteria);
		return CollectionUtil.isEmpty(objectNodes) ? null : objectNodes.get(0);
	}
	
	/**
	 * 获取BimId
	 * 
	 * @param objectNode
	 * @return
	 */
	private String getObjectBimId(ObjectNode objectNode) {
		String infos = objectNode.has("infos") ? objectNode.get("infos").asText() : null;
		if (StrUtil.isNotBlank(infos)) {
			try {
				JSONObject jsonObject = JSONObject.parseObject(infos);
				return jsonObject.getString("bimId");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return null;
	}
	
}
