package com.persagy.proxy.report.service.impl;

import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.persagy.dmp.basic.dto.GraphParentMes;
import com.persagy.dmp.basic.dto.RequestData;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.context.AppContext;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.define.client.DigitalDefineFacade;
import com.persagy.dmp.define.client.DigitalGraphDefineFacade;
import com.persagy.dmp.define.client.DigitalObjectInfoFacade;
import com.persagy.dmp.define.client.DigitalRelationDefineFacade;
import com.persagy.dmp.define.entity.GraphDefine;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import com.persagy.dmp.define.entity.ObjectTypeDefine;
import com.persagy.dmp.define.entity.RelationDefine;
import com.persagy.dmp.digital.client.DigitalObjectFacade;
import com.persagy.dmp.digital.client.DigitalRelationFacade;
import com.persagy.dmp.digital.client.DigitalRelationProjectCalFacade;
import com.persagy.dmp.digital.entity.ObjectRelation;
import com.persagy.dmp.digital.entity.ObjectRelationProjectCal;
import com.persagy.dmp.simple.client.DigitalObjectSimpleFacade;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.adm.constant.SpaceTypeEnum;
import com.persagy.proxy.report.service.IRelationReportService;
import com.persagy.proxy.adm.strategy.relationdata.RelationObjectStrategy;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.EnumUtil;
import cn.hutool.core.util.StrUtil;

/**
 * 报表/模板下载
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年8月31日 下午4:43:15
 */
@Service
public class RelationReportService implements IRelationReportService {

	@Value("${middleware.group.code}")
	private String defaultCode;

	@Override
	public List<ObjectTypeDefine> queryClassList(String groupCode, String projectId, Set<String> classCodes, Set<String> objTypes, List<OrderItem> orderItems) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria classCriteria = new QueryCriteria();
		ObjectNode condition = objectMapper.createObjectNode();
		if (CollectionUtil.isNotEmpty(classCodes)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", classCodes);
			condition.putPOJO("code", arrays);
		}
		if (CollectionUtil.isNotEmpty(objTypes)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", objTypes);
			condition.putPOJO("objType", arrays);
		}
		condition.put("valid", 1);
		classCriteria.setCriteria(condition);
		if (CollectionUtil.isNotEmpty(orderItems)) {
			classCriteria.setOrders(orderItems);
		}
		classCriteria.setPage(1L);
		classCriteria.setSize(500L);
		List<ObjectTypeDefine> all = new ArrayList<ObjectTypeDefine>();
		this.queryPageClass(all, classCriteria, groupCode, projectId);
		return all;
	}
	
	@Override
	public List<Map<String, Object>> queryObjectCountByObjInfos(String groupCode, Set<String> classCodes, Set<String> objTypes, List<String> groups) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		RequestData requestData = new RequestData();
		requestData.setClassCodes(classCodes);
		requestData.setObjTypes(objTypes);
		requestData.setGroups(groups);
		List<Map<String,Object>> list = DigitalObjectSimpleFacade.queryObjectCountByObjInfos(groupCode, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, requestData);
		return CollectionUtil.isEmpty(list) ? Lists.newArrayList() : list;
	}
	
	@Override
	public List<GraphDefine> getGraphDefine(String groupCode, String projectId, String graphCode) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		if (StrUtil.isNotBlank(graphCode)) {
			criteria.put("code", graphCode);
		}
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		List<GraphDefine> graphDefines = DigitalGraphDefineFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return graphDefines;
	}
	
	@Override
	public List<RelationDefine> getRelationDefine(String groupCode, String projectId, String graphCode, String relCode) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		if (StrUtil.isNotBlank(graphCode)) {
			criteria.put("graphCode", graphCode);
		}
		if (StrUtil.isNotBlank(relCode)) {
			criteria.put("code", relCode);
		}
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		List<RelationDefine> relationDefines = DigitalRelationDefineFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return relationDefines;
	}

	@Override
	public List<JSONObject> getAllRelationProjectCal(QueryCriteria queryCriteria, String groupCode, String projectId) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;

		// 查询出对应项目关系计算数据
		List<ObjectNode> projectList = DigitalRelationProjectCalFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (CollectionUtil.isEmpty(projectList)) {
			return Lists.newArrayList();
		}
		List<JSONObject> result = Lists.newArrayList();
		Map<String, AdmRelationTypeEnum> graphRelTypeMap = AdmCommonConstant.GRAPH_RELATION_TYPE_MAP;
		Map<String, RelationDefine> relationDefineMap = queryAllRelationDefineMap(groupCode, projectId);
		for (ObjectNode objectNode : projectList) {
			String graphCode = objectNode.get("graphCode") == null ? null : objectNode.get("graphCode").asText();
			String relCode = objectNode.get("relCode") == null ? null : objectNode.get("relCode").asText();
			//AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.GRAPH_RELATION_TYPE_MAP.get(graphCode + AdmCommonConstant.UNDERLINE + relCode);
			result.add(this.fillRelationProjectCal(objectNode, graphCode, relCode, graphRelTypeMap, relationDefineMap, groupCode, projectId));
		}
		return result;
	}

	@Override
	public List<JSONObject> getAllRelationProjectCalTree(String groupCode, String projectId) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;

		// 获取所有的二级标签
		QueryCriteria secondCriteria = new QueryCriteria();
		List<GraphDefine> secondList = DigitalGraphDefineFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, secondCriteria);
		if (CollectionUtil.isEmpty(secondList)) {
			return Lists.newArrayList();
		}
		// 获取此图类型对应项目计算数据
		List<JSONObject> resultList = new ArrayList<>();
		Map<String, AdmRelationTypeEnum> graphRelTypeMap = AdmCommonConstant.GRAPH_RELATION_TYPE_MAP;
		Map<String, RelationDefine> relationDefineMap = queryAllRelationDefineMap(groupCode, projectId);
		for (GraphDefine graphDefine : secondList) {
			String graphCode = graphDefine.getCode();
			JSONObject graphObject = this.transfer(graphDefine);

			// 查询出对应项目关系计算数据
			QueryCriteria projectCriteria = new QueryCriteria();
			ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
			ObjectNode criteria = objectMapper.createObjectNode();
			criteria.put("projectId", projectId);
			criteria.put("graphCode", graphCode);
			criteria.put("valid", 1);
			projectCriteria.setCriteria(criteria);
			List<ObjectNode> projectList = DigitalRelationProjectCalFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, projectCriteria);
			if (CollectionUtil.isEmpty(projectList)) {
				graphObject.put("relationTypeProjectList", Lists.newArrayList());
				continue;
			}
			// 再根据图类型、边类型，统计此类型下的数量
			List<JSONObject> tempList = new ArrayList<>();
			for (ObjectNode objectNode : projectList) {
				String relCode = objectNode.get("relCode") == null ? null : objectNode.get("relCode").asText();
				tempList.add(this.fillRelationProjectCal(objectNode, graphDefine.getCode(), relCode, graphRelTypeMap,relationDefineMap, groupCode, projectId));
			}
			graphObject.put("relationTypeProjectList", tempList);
			resultList.add(graphObject);
		}

		return resultList;
	}

	@Override
	public JSONObject getRelationProjectCal(String groupCode, String projectId, String relType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
		if (relationTypeEnum == null) {
			return null;
		}
		
		ObjectNode objectNode = this.getRelationProjectCal(groupCode, projectId, relationTypeEnum.getGraphCode(),
				relationTypeEnum.getRelCode(), null);
		Map<String, AdmRelationTypeEnum> graphRelTypeMap = AdmCommonConstant.GRAPH_RELATION_TYPE_MAP;
		Map<String, RelationDefine> relationDefineMap = this.queryAllRelationDefineMap(groupCode, projectId);
		if (objectNode != null) {
			return this.fillRelationProjectCal(objectNode, relationTypeEnum.getGraphCode(), relationTypeEnum.getRelCode(), graphRelTypeMap, relationDefineMap, groupCode, projectId);
		}
		
		return new JSONObject();
	}
	
	@Override
	public ObjectNode getRelationProjectCal(String groupCode, String projectId, String graphCode, String relCode, String zoneType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		if (StrUtil.isNotBlank(zoneType)) {
			criteria.put("zoneType", zoneType);
		}
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		List<ObjectNode> objectNodes = DigitalRelationProjectCalFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (CollectionUtil.isEmpty(objectNodes)) {
			return null;
		}
		return objectNodes.get(0);
	}
	
	@Override
	public long countRelationObject(String groupCode, String projectId, String graphCode, String relCode, String zoneType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		if (StrUtil.isNotBlank(zoneType)) {
			criteria.put("relValue", zoneType);
		}
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		queryCriteria.setOnlyCount(true);
		CommonResult<List<ObjectRelation>> result = DigitalRelationFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return result == null ? 0 : (result.getCount() == null ? 0 : result.getCount());
	}
	
	@Override
	public long countObject(String groupCode, String projectId, String classCode, String objType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("classCode", classCode);
		if (StrUtil.isNotBlank(objType)) {
			criteria.put("objType", objType);
		}
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		queryCriteria.setOnlyCount(true);
		CommonResult<List<ObjectNode>> result = DigitalObjectFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return result == null ? 0 : (result.getCount() == null ? 0 : result.getCount());
	}
	
	@Override
	public List<ObjectNode> getObjectNode(ObjectNode criteria, String groupCode, String projectId, Set<String> ids) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		QueryCriteria queryCriteria = new QueryCriteria();
		if (criteria == null) {
			ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
			criteria = objectMapper.createObjectNode();
		}
		if (CollectionUtil.isNotEmpty(ids)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", ids);
			criteria.putPOJO("id", arrays);
		}
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		List<ObjectNode> objectNodes = DigitalObjectFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return objectNodes;
	}
	
	@Override
	public List<ObjectNode> findObjectNodes(String groupCode, String projectId, String objType, String code, String value) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		if (StrUtil.isBlank(code)) {
			return null;
		}
		String key = RelationObjectStrategy.OBJNAME_2_TYPE.get(code);
		if (StrUtil.isBlank(key)) {
			return null;
		}
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put(key, value);
		criteria.put("objType", objType);
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		queryCriteria.setPage(1L);
		queryCriteria.setSize(50000L);
		
		List<ObjectNode> all = new ArrayList<ObjectNode>();
		this.queryPageObjects(all, queryCriteria, groupCode, projectId);
		return all;
	}
	
	@Override
	public ObjectNode getObjectByCodeAndId(String groupCode, String projectId, String graphCode, String relCode, String objFrom, String objTo) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		// 获取出关联对象
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		if (StrUtil.isNotBlank(objFrom)) {
			criteria.put("objFrom", objFrom);
		}
		if (StrUtil.isNotBlank(objTo)) {
			criteria.put("objTo", objTo);
		}
		criteria.put("valid", 1);
		queryCriteria.setCriteria(criteria);
		List<ObjectRelation> objectRelations = DigitalRelationFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (CollectionUtil.isEmpty(objectRelations)) {
			return objectMapper.createObjectNode();
		}
		
		// 获取关联对象的本地名称
		ObjectRelation objectRelation = objectRelations.get(0);
		String relationId = StrUtil.isBlank(objFrom) ? objectRelation.getObjFrom() : objectRelation.getObjTo();
		List<ObjectNode> objectNodes = this.getObjectNode(null, groupCode, projectId, Sets.newHashSet(relationId));
		return CollectionUtil.isEmpty(objectNodes) ? objectMapper.createObjectNode() : objectNodes.get(0);
	}

	@Override
	public List<ObjectNode> queryObjects(ObjectNode criteria, String groupCode, String projectId, Set<String> ids, 
			Set<String> classCodes, Set<String> objTypes, List<OrderItem> orderItem) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria objectCriteria = new QueryCriteria();
		if (criteria == null) {
			criteria = objectMapper.createObjectNode();
		}
		if (CollectionUtil.isNotEmpty(ids)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", ids);
			criteria.putPOJO("id", arrays);
		}
		if (CollectionUtil.isNotEmpty(classCodes)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", classCodes);
			criteria.putPOJO("classCode", arrays);
		}
		if (CollectionUtil.isNotEmpty(objTypes)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", objTypes);
			criteria.putPOJO("objType", arrays);
		}
		criteria.put("valid", 1);
		objectCriteria.setPage(1L);
		objectCriteria.setSize(500L);
		objectCriteria.setCriteria(criteria);
		if (CollectionUtil.isNotEmpty(orderItem)) {
			objectCriteria.setOrders(orderItem);
		}
		List<ObjectNode> all = new ArrayList<ObjectNode>();
		this.queryPageObjects(all, objectCriteria, groupCode, projectId);
		return all;
	}
	
	@Override
	public List<ObjectRelation> findRelationObjects(String groupCode, String projectId, String graphCode, String relCode, Set<String> objFroms, Set<String> objTos) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		// 获取出关联对象
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		if (StrUtil.isNotBlank(graphCode)) {
			criteria.put("graphCode", graphCode);
		}
		if (StrUtil.isNotBlank(relCode)) {
			criteria.put("relCode", relCode);
		}
		if (CollectionUtil.isNotEmpty(objFroms)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", objFroms);
			criteria.putPOJO("objFrom", arrays);
		}
		if (CollectionUtil.isNotEmpty(objTos)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", objTos);
			criteria.putPOJO("objTo", arrays);
		}
		criteria.put("valid", 1);
		queryCriteria.setPage(1L);
		queryCriteria.setSize(500L);
		queryCriteria.setCriteria(criteria);
		
		List<ObjectRelation> all = new ArrayList<ObjectRelation>();
		this.queryPageRelations(all, queryCriteria, groupCode, projectId);
		return all;
	}
	
	@Override
	public List<ObjectInfoDefine> queryFunidList(String groupCode, String projectId, Set<String> classCodes, String code, String category) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode condition = objectMapper.createObjectNode();
		if (StrUtil.isNotBlank(code)) {
			condition.put("code", code);
		}
		if (StrUtil.isNotBlank(category)) {
			condition.put("category", category);
		}
		if (CollectionUtil.isNotEmpty(classCodes)) {
			JSONObject arrays = new JSONObject();
			arrays.put("$in", classCodes);
			condition.putPOJO("classCode", arrays);
		}
		condition.put("valid", 1);
		queryCriteria.setCriteria(condition);
		queryCriteria.setPage(1L);
		queryCriteria.setSize(500L);
		List<ObjectInfoDefine> all = new ArrayList<ObjectInfoDefine>();
		this.queryPageFunids(all, queryCriteria, groupCode, projectId);
		return all;
	}
	
	@Override
	public boolean createRelationProjectCal(String groupCode, String projectId) throws JsonProcessingException {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		// 1.验证是否已经添加过
		QueryCriteria queryCriteria = new QueryCriteria();
		queryCriteria.setOnlyCount(true);
		CommonResult<List<ObjectNode>> prototype = DigitalRelationProjectCalFacade.queryPrototype(groupCode, projectId,
				AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (prototype != null && prototype.getCount() != null && prototype.getCount() > 0) {
			return true;
		}
		
		// 2.获取所有的边类型
		QueryCriteria criteria = new QueryCriteria();
		List<RelationDefine> list = DigitalRelationDefineFacade.query(groupCode, projectId,
				AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, criteria);
		if (CollectionUtil.isEmpty(list)) {
			return true;
		}
		
		// 3.组装关系计算数据
		List<ObjectNode> nodes = new ArrayList<ObjectNode>();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		for (RelationDefine relationDefine : list) {
			// 3.1 判断是否需要加入项目
			if (null==relationDefine.getCalFlag()
					|| !relationDefine.getCalFlag()){
				continue;
			}
			// 3.2 判断zoneTypes是否为多个,如果多个则根据zoneType创建
			if (CollUtil.isNotEmpty(relationDefine.getZoneTypes())){
				for (String zoneType : relationDefine.getZoneTypes()) {
					nodes.add(createRelationProjectCalInternal(objectMapper,groupCode,projectId,relationDefine,zoneType));
				}
				continue;
			}
			// 3.3 如果没有分区则直接创建一个
			nodes.add(createRelationProjectCalInternal(objectMapper,groupCode,projectId,relationDefine,null));
		}
		List<ObjectRelationProjectCal> result = DigitalRelationProjectCalFacade.create(groupCode, projectId,
				AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, nodes);
		return CollectionUtil.isNotEmpty(result);
	}
	
	@Override
	public JSONObject overview(String content) throws JsonProcessingException {
		JSONObject result = new JSONObject();
		if (StrUtil.isBlank(content)) {
			this.createRelationProjectCal(AppContext.getContext().getGroupCode(), AppContext.getContext().getProjectId());
		}
		List<JSONObject> parentList = new ArrayList<>();
		// 获取树结构
		this.getGraphRelationProjectTree(parentList, AppContext.getContext().getGroupCode(),
				AppContext.getContext().getProjectId(), content);
		result.put("content", parentList);
		result.put("message", "");
		result.put("pageNumber", 1);
		result.put("pageSize", 1000);
		result.put("result", "success");
		result.put("total", parentList.size());
		return result;
	}
	
	/***
	 * Description: 创建存储的对象
	 * @param groupCode : 集团编码
	 * @param projectId : 项目id
	 * @param relationDefine : 关系定义
	 * @param zoneType : 空间分区类型
	 * @return : void
	 * @author : lijie
	 * @date :2021/10/14 16:14
	 * Update By lijie 2021/10/14 16:14
	 */
	private ObjectNode createRelationProjectCalInternal(ObjectMapper objectMapper, String groupCode,
												  		String projectId, RelationDefine relationDefine,
														String zoneType) throws JsonProcessingException {
		ObjectNode objectNode = objectMapper.createObjectNode();
		objectNode.put("groupCode", groupCode);
		objectNode.put("projectId", projectId);
		objectNode.put("id", IdWorker.getIdStr());
		objectNode.put("graphCode", relationDefine.getGraphCode());
		objectNode.put("relCodeName", relationDefine.getName());
		objectNode.put("relCode", relationDefine.getCode());
		objectNode.put("sourceFlag", relationDefine.getSourceFlag());	// 是否需要配置源末端
		objectNode.put("automaticFlag", relationDefine.getAutomaticFlag());	// 是否自动计算类型
		objectNode.put("manual", relationDefine.getManual());	// 手动计算类型 1 禁用  2提示引导 3手动维护
		objectNode.put("computationalState", 1);	// 计算状态 1初始状态 2需要计算 3计算中 4等待中 5计算失败（如果处于计算中状态，则不再次调用）
		objectNode.put("zoneType", zoneType);	// 业务分区类型
		objectNode.put("systemType", relationDefine.getSystemType());	// 风系统1,水系统0
		objectNode.put("computeVersion", 0);	// 计算版本
		objectNode.putPOJO("mepSystemType", CollUtil.isNotEmpty(relationDefine.getMepSystemType())
						? objectMapper.valueToTree(relationDefine.getMepSystemType()) :objectMapper.createArrayNode());	// 机电系统类型
		objectNode.put("creator", AdmCommonConstant.USER_ID);	// 创建人
		objectNode.put("modifier", AdmCommonConstant.USER_ID);	// 最后修改人
		return objectNode;
	}
	
	/**
	 * 获取图类型下的边类型的项目计算关系数据，并形成三级树结构
	 * 
	 * @param parentList
	 * @param groupCode
	 * @param projectId
	 * @param content
	 */
	private void getGraphRelationProjectTree(List<JSONObject> parentList, String groupCode, String projectId, String content) {
		//boolean isQuery = StringUtils.isNotBlank(content);
		// 1.获取所有的一级,二级图类型定义
		QueryCriteria parentCriteria = new QueryCriteria();
		List<GraphDefine> graphDefines = DigitalGraphDefineFacade.query(groupCode, projectId,
				AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, parentCriteria);
		if (CollUtil.isEmpty(graphDefines)){
			return;
		}
		// 2.查询所有的边类型定义
		Map<String, RelationDefine> relationDefineMap = queryAllRelationDefineMap(groupCode,projectId);
		// 3.查询出对应项目关系计算数据
		List<ObjectNode> projectRelList = queryRelationProjectCalsByProjectId(groupCode,projectId,content);
		if (CollUtil.isEmpty(projectRelList)){
			return;
		}
		Map<String,List<JSONObject>> levelMap = new HashMap<>();
		Map<String, AdmRelationTypeEnum> graphRelTypeMap = AdmRelationTypeEnum.getGraphRelTypeMap();
		for (ObjectNode projectRel : projectRelList) {
			if (!projectRel.has("relCode")
					|| StrUtil.isBlank(projectRel.get("relCode").asText())){
				continue;
			}
			String relCode = projectRel.get("relCode").asText();
			String graphCode = projectRel.get("graphCode").asText();
			if (!graphRelTypeMap.containsKey(graphCode+StrUtil.UNDERLINE+relCode)){
				continue;
			}
			List<JSONObject> sonList = levelMap.getOrDefault(graphCode, new ArrayList<>());
			JSONObject son = this.fillRelationProjectCal(projectRel, graphCode,relCode,graphRelTypeMap,relationDefineMap, null, null);
			sonList.add(son);
			sonList.sort(Comparator.comparing(b -> b.getString("relationTypeCode")));
			levelMap.put(graphCode,sonList);
		}
		// 3.使用图类型定义的二级标签作为第一级
		List<JSONObject> topList = new ArrayList<>();
		Set<String> categoryIds = new HashSet<>();
		for (GraphDefine graphDefine : graphDefines) {
			GraphParentMes secondCategoryMes = graphDefine.getSecondCategoryMes();
			if (null==secondCategoryMes){
				continue;
			}
			if (!categoryIds.contains(secondCategoryMes.getId())){
				categoryIds.add(secondCategoryMes.getId());
				topList.add(this.transfer(secondCategoryMes));
			}
			List<JSONObject> threeLevelList = levelMap.getOrDefault(graphDefine.getCode(), new ArrayList<>());
			if (CollUtil.isEmpty(threeLevelList)){
				continue;
			}
			List<JSONObject> sonList = levelMap.getOrDefault(secondCategoryMes.getId(), new ArrayList<>());
			JSONObject secondLevel = this.transfer(graphDefine);
			secondLevel.put("relationTypeProjectList",threeLevelList);
			sonList.add(secondLevel);
			sonList.sort(Comparator.comparing(b -> b.getString("graphTypeCode")));
			levelMap.put(secondCategoryMes.getId(),sonList);
		}
		if (CollUtil.isEmpty(topList)){
			return;
		}
		for (int i = topList.size() - 1; i >= 0; i--) {
			JSONObject top = topList.get(i);
			List<JSONObject> secondLevelList = levelMap.getOrDefault(top.getString("graphTypeId"), new ArrayList<>());
			if (CollUtil.isEmpty(secondLevelList)){
				topList.remove(i);
				continue;
			}
			top.put("childGraphicTypeList",secondLevelList);
		}
		// 顶级按sort排序
		topList.sort(Comparator.comparing(b -> b.getIntValue("sort")));
		parentList.addAll(topList);
	}
	
	/***
	 * Description: 根据项目id查询项目计算关系
	 * @param groupCode : 集团编码
	 * @param projectId : 项目id
	 * @param content : 查询关键字
	 * @return : java.util.List<com.fasterxml.jackson.databind.node.ObjectNode>
	 * @author : lijie
	 * @date :2021/10/15 11:26
	 * Update By lijie 2021/10/15 11:26
	 */
	private List<ObjectNode> queryRelationProjectCalsByProjectId(String groupCode, String projectId,String content) {
		QueryCriteria projectCriteria = new QueryCriteria();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("projectId", projectId);
		if (StrUtil.isNotBlank(content)) {
			ObjectNode condition = objectMapper.createObjectNode();
			condition.put("$like", content);
			criteria.putPOJO("name", condition); // 模糊查询
		}
		criteria.put("valid", 1);
		projectCriteria.setCriteria(criteria);
		projectCriteria.setOrders(Collections.singletonList(OrderItem.asc("relCode")));
		return DigitalRelationProjectCalFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, projectCriteria);
	}

	/***
	 * Description: 查询所有关系类型
	
	 * @return : java.util.Map<java.lang.String,com.persagy.dmp.define.entity.RelationDefine>  
	 * @author : lijie
	 * @date :2021/10/15 11:22
	 * Update By lijie 2021/10/15 11:22
	 */
	private Map<String, RelationDefine> queryAllRelationDefineMap(String groupCode,String projectId) {
		List<RelationDefine> relationDefines = DigitalRelationDefineFacade.query(groupCode, projectId,
				AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, new QueryCriteria());
		if (CollUtil.isEmpty(relationDefines)){
			return new HashMap<>();
		}
		return relationDefines.stream()
				.collect(Collectors.toMap(relationDefine ->
								relationDefine.getGraphCode() + StrUtil.UNDERLINE + relationDefine.getCode(),
						relationDefine -> relationDefine, (k1, k2) -> k1));
	}

	/***
	 * Description: 创建一级返回数据
	 * @param graphParentMes : 一级信息对象
	 * @return : com.alibaba.fastjson.JSONObject
	 * @author : lijie
	 * @date :2021/10/14 20:56
	 * Update By lijie 2021/10/14 20:56
	 */
	private JSONObject transfer(GraphParentMes graphParentMes) {
		JSONObject top = new JSONObject();
		top.put("sort", graphParentMes.getSort());
		top.put("note", "");
		top.put("statistics", new JSONObject());
		top.put("graphTypeName", graphParentMes.getName());
		top.put("graphTypeId", graphParentMes.getId());
		top.put("graphTypeCode", graphParentMes.getCode());
		top.put("code", graphParentMes.getCode());
		top.put("lastUpdate", DateUtil.format(new Date(),"yyyy-MM-dd HH:mm:ss"));
		top.put("createTime", DateUtil.format(new Date(),"yyyy-MM-dd HH:mm:ss"));
		return top;
	}

	/**
	 * 获取转换后的数据，图类型数据转换
	 * 
	 * @param graphDefine
	 * @return
	 */
	private JSONObject transfer(GraphDefine graphDefine) {
		JSONObject top = new JSONObject();
		top.put("sort", 10);
		top.put("note", "");
		top.put("statistics", new JSONObject());
		top.put("graphTypeName", graphDefine.getName());
		top.put("graphTypeId", graphDefine.getId());
		top.put("graphTypeCode", graphDefine.getCode());
		top.put("code", graphDefine.getCode());
		top.put("lastUpdate", graphDefine.getModifiedTime() == null ? null : graphDefine.getModifiedTime().toString(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
		top.put("createTime", graphDefine.getCreationTime() == null ? null : graphDefine.getCreationTime().toString(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
		return top;
	}
	
	/**
	 * 获取转换后的数据，项目计算数据转换
	 * 
	 * @param relationProject :项目对象
	 * @param relCode :边类型编码
	 * @param graphRelTypeMap :与旧数据中心的code的映射关系
	 * @param relationDefineMap
	 * @return
	 */
	private JSONObject fillRelationProjectCal(ObjectNode relationProject, String graphicId, String relCode,
								Map<String, AdmRelationTypeEnum> graphRelTypeMap,
								Map<String, RelationDefine> relationDefineMap,
								String groupCode, String projectId) {
		String key = graphicId + StrUtil.UNDERLINE + relCode;
		AdmRelationTypeEnum typeEnum = graphRelTypeMap.get(key);
		RelationDefine relationDefine = relationDefineMap.getOrDefault(key,new RelationDefine());
		JSONObject result = new JSONObject();
		long count = 0;
		if (StrUtil.isAllNotBlank(groupCode, projectId)) {
			String zoneType = relationProject.has("zoneType") ? relationProject.get("zoneType").asText() : null;
			count = this.countRelationObject(groupCode, projectId, graphicId, relCode, zoneType);
		}
		
		result.put("count", count);
		result.put("graphicId", graphicId);
		result.put("statistics", new JSONObject());
		result.put("conneObject", relationDefine.getTargetObjs());
		
		result.put("computingTime", relationProject.has("computingTime") ? DateUtil.parse(relationProject.get("computingTime").asText(), "yyyyMMddHHmmss").toString() : null);
		result.put("computationalState",relationProject.has("computationalState")?relationProject.get("computationalState").asText():null);
		result.put("id",relationProject.has("id")?relationProject.get("id").asText():null);
		result.put("manual",relationProject.has("manual")?relationProject.get("manual").asLong():null);
		result.put("projectId",relationProject.has("projectId")?relationProject.get("projectId").asText():null);
		result.put("relationType", typeEnum == null ? null : typeEnum.getRelationType());
		result.put("relManualType",typeEnum == null ? null : typeEnum.getRelationType());
		result.put("relationTypeCode",relationProject.has("relCode")?relationProject.get("relCode").asText():null);
		result.put("code", relationProject.has("relCode")?relationProject.get("relCode").asText():null);
		result.put("relationTypeName",relationProject.has("relCodeName")?relationProject.get("relCodeName").asText():null);
		result.put("source", relationProject.has("sourceFlag") && relationProject.get("sourceFlag").asBoolean());
		result.put("automatic", relationProject.has("automaticFlag") && relationProject.get("automaticFlag").asBoolean());
		result.put("mepSystemType", relationProject.has("mepSystemType") ? relationProject.get("mepSystemType").asText() : new JSONArray());
		
		Long modifiedTime = relationProject.get("modifiedTime") == null ? null : relationProject.get("modifiedTime").asLong();
		Long creationTime = relationProject.get("creationTime") == null ? null : relationProject.get("creationTime").asLong();

		result.put("lastUpdate", modifiedTime == null ? null : new DateTime(modifiedTime).toString());
		result.put("createTime", creationTime == null ? null : new DateTime(creationTime).toString());
		result.put("mainObject", relationDefine.getFromObjType());
		result.put("fromObject", relationDefine.getToObjType());
		result.put("prompting", relationDefine.getPrompting());
		// 空间分区
		String zoneType = relationProject.has("zoneType") ? relationProject.get("zoneType").asText() : null;
		if (StrUtil.isNotBlank(zoneType)){
			SpaceTypeEnum spaceTypeEnum = EnumUtil.fromString(SpaceTypeEnum.class, zoneType);
			if (null!=spaceTypeEnum){
				result.put("zoneType",spaceTypeEnum.getCode());
				result.put("note",relationDefine.getRemark());
				result.put("relationTypeName",StrUtil.replace(result.getString("relationTypeName"),"业务空间",spaceTypeEnum.getDesc()));
			}
		}
		return result;
	}

	
	/***************************************************** 所有接口的额外分页查询 **********************************************/
	
	/**
	 * 分页查询类定义数据
	 * 
	 * @param all
	 * @param classCriteria
	 * @param groupCode
	 * @param projectId
	 */
	private void queryPageClass(List<ObjectTypeDefine> all, QueryCriteria classCriteria, String groupCode, String projectId) {
		CommonResult<List<ObjectTypeDefine>> queryPrototype = DigitalDefineFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, classCriteria);
		Long count = queryPrototype.getCount();
		List<ObjectTypeDefine> infos = queryPrototype.getData();
		if (count == null || CollectionUtil.isEmpty(infos)) {
			return;
		}
		
		all.addAll(infos);
		
		if (count < 500) {
			return;
		} else if ((infos.size()*classCriteria.getPage()) > count) {
			return;
		} else {
			classCriteria.setPage((classCriteria.getPage() + 1));
			this.queryPageClass(all, classCriteria, groupCode, projectId);
		}
	}
	
	/**
	 * 分页查询对象数据
	 * 
	 * @param all
	 * @param objectCriteria
	 * @param groupCode
	 * @param projectId
	 */
	private void queryPageObjects(List<ObjectNode> all, QueryCriteria objectCriteria, String groupCode, String projectId) {
		CommonResult<List<ObjectNode>> queryPrototype = DigitalObjectFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, objectCriteria);
		Long count = queryPrototype.getCount();
		List<ObjectNode> infos = queryPrototype.getData();
		if (count == null || CollectionUtil.isEmpty(infos)) {
			return;
		}
		
		all.addAll(infos);
		
		if (count < 500) {
			return;
		} else if ((infos.size()*objectCriteria.getPage()) > count) {
			return;
		} else {
			objectCriteria.setPage((objectCriteria.getPage() + 1));
			this.queryPageObjects(all, objectCriteria, groupCode, projectId);
		}
	}

	/**
	 * 分页查询关系数据
	 * 
	 * @param all
	 * @param queryCriteria
	 * @param groupCode
	 * @param projectId
	 */
	private void queryPageRelations(List<ObjectRelation> all, QueryCriteria queryCriteria, String groupCode, String projectId) {
		CommonResult<List<ObjectRelation>> queryPrototype = DigitalRelationFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		Long count = queryPrototype.getCount();
		List<ObjectRelation> infos = queryPrototype.getData();
		if (count == null || CollectionUtil.isEmpty(infos)) {
			return;
		}
		
		all.addAll(infos);
		
		if (count < 500) {
			return;
		} else if ((infos.size()*queryCriteria.getPage()) > count) {
			return;
		} else {
			queryCriteria.setPage((queryCriteria.getPage() + 1));
			this.queryPageRelations(all, queryCriteria, groupCode, projectId);
		}
	}
	
	/**
	 * 分页查询信息点定义数据
	 * 
	 * @param all
	 * @param queryCriteria
	 * @param groupCode
	 * @param projectId
	 */
	private void queryPageFunids(List<ObjectInfoDefine> all, QueryCriteria queryCriteria, String groupCode, String projectId) {
		CommonResult<List<ObjectInfoDefine>> queryPrototype = DigitalObjectInfoFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		Long count = queryPrototype.getCount();
		List<ObjectInfoDefine> infos = queryPrototype.getData();
		if (count == null || CollectionUtil.isEmpty(infos)) {
			return;
		}
		
		all.addAll(infos);
		
		if (count < 500) {
			return;
		} else if ((infos.size()*queryCriteria.getPage()) > count) {
			return;
		} else {
			queryCriteria.setPage((queryCriteria.getPage() + 1));
			this.queryPageFunids(all, queryCriteria, groupCode, projectId);
		}
	}
	
}
